﻿using Fw.Utility.UnifyResponse;
using Hotline.DataSharing.Order;
using Hotline.DataSharing.Province.Notifications;
using Hotline.DataSharing.WebPortal;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.DataSharing.WebPortal;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.DataSharing;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using MongoDB.Driver.Linq;
using Sharing.Share.Dtos.DataSharing.WebPortal;
using SqlSugar;
using XF.Domain.Filters;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers.DataSharing
{
    /// <summary>
    /// 门户网站数据获取
    /// </summary>
    public class WebPortalController : BaseController
    {
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;
        private readonly IRepository<DsBulletin> _bulletinRepository;
        private readonly IRepository<DsOrder> _dataOrderRepository;
        private readonly IRepository<DsWebFlowAccept> _webFlowAcceptRepository;
        private readonly IRepository<DsWebUserRegister> _webUserRegisterRepository;
        private readonly IRepository<DsWebUserAuth> _webUserAuthRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="mediator"></param>
        /// <param name="bulletinRepository"></param>
        /// <param name="dataOrderRepository"></param>
        /// <param name="webFlowAcceptRepository"></param>
        /// <param name="webUserRegisterRepository"></param>
        /// <param name="webUserAuthRepository"></param>
        public WebPortalController(IMapper mapper, IMediator mediator,
            IRepository<DsBulletin> bulletinRepository,
               IRepository<DsOrder> dataOrderRepository,
            IRepository<DsWebFlowAccept> webFlowAcceptRepository,
              IRepository<DsWebUserRegister> webUserRegisterRepository,
            IRepository<DsWebUserAuth> webUserAuthRepository)
        {
            _mapper = mapper;
            _mediator = mediator;
            _bulletinRepository = bulletinRepository;
            _dataOrderRepository = dataOrderRepository;
            _webFlowAcceptRepository = webFlowAcceptRepository;
            _webUserRegisterRepository = webUserRegisterRepository;
            _webUserAuthRepository = webUserAuthRepository;

        }

        #region 通知
        /// <summary>
        /// 获取列表
        /// </summary>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getarticlelist")]
        public async Task<OpenResponse> GetArticleList([FromBody] QueryArticleListDto dto)
        {
            Kv kv = new() { Key = "2", Value = "门户网站" };
            RefAsync<int> total = 0;
            var items = await _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }))
                 .Where(p => p.BulletinTypeId == dto.NoticeType)
                 .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                 .OrderByDescending(p => p.BulletinTime)
                 .Select(it => new
                 {
                     Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                     NoticeID = it.Id,
                     NoticeContent = it.Content,
                     NoticeTypeName = it.BulletinTypeName,
                     NoticeTitle = it.Title,
                     NoticeBMName = it.SourceOrgName,
                     NoticeCreateDate = it.BulletinTime,
                     VideoUrl = "",
                     WNLT_FullCode = ""
                 })
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));
            ArticleListDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<ArticleListDto>>(items)
            };

            return OpenResponse.Ok(WebPortalDeResponse<ArticleListDataDto>.Success(dataDto));
        }

        /// <summary>
        /// 获取详情，修改阅读次数
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getarticledetails")]
        public async Task<OpenResponse> GetArticleDetails(string id)
        {
            var data = await _bulletinRepository.GetAsync(p => p.Id == id, HttpContext.RequestAborted);

            if (data != null)
            {
                data.ReadedNum = data.ReadedNum++;
                await _bulletinRepository.UpdateAsync(data, HttpContext.RequestAborted);
            }

            ArticleDetailsDto detailsDto = new()
            {
                NoticeID = data.Id,
                NoticeTypeName = data.BulletinTypeName,
                NoticeTitle = data.Title,
                NoticeBMName = data.SourceOrgName,
                NoticeCreateDate = data.BulletinTime,
                NoticeRCount = data.ReadedNum,
                WNED_VideoUrl = "",
                NoticeContent = data.Content
            };

            return OpenResponse.Ok(WebPortalDeResponse<ArticleDetailsDto>.Success(detailsDto));
        }

        /// <summary>
        /// 上一条和下一条
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getpreviousandnext")]
        public async Task<OpenResponse> GetPreviousAndNext([FromBody] ArticlePreviousAndNextDto dto)
        {
            var sugar = _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }));
            if (dto.FullSearch == "1")//全文搜索
            {
                sugar.Where(p => p.BulletinTypeId == "1" || p.BulletinTypeId == "5" || p.BulletinTypeId == "6" || p.BulletinTypeId == "7" || p.BulletinTypeId == "3")
                    .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Content.Contains(dto.Condition) || p.Title.Contains(dto.Condition));
            }
            else//指定分类
            {
                sugar.WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                    .Where(p => p.BulletinTypeId == dto.NoticeTypeId);
            }

            var list = await sugar.OrderByDescending(p => p.BulletinTime)
                .Select(it => new
                {
                    Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                    NoticeID = it.Id,
                    NoticeTitle = it.Title,
                    NoticeCreateDate = it.BulletinTime
                })
                .ToListAsync();
            //数据为空返回空数据
            if (list == null || list.Count == 0)
                return OpenResponse.Ok(WebPortalDeResponse<List<ArticlePreviousAndNextDataDto>>.Success(null));
            else
            {
                var temp = list.Find(p => p.NoticeID == dto.ID);
                if (temp != null)
                {
                    List<ArticlePreviousAndNextDataDto> returnDto = new();
                    //上一条
                    var pTemp = list.Find(p => p.Page == temp.Page - 1);
                    if (pTemp != null)

                        returnDto.Add(new()
                        {
                            NoticeID = pTemp.NoticeID,
                            NoticeTitle = pTemp.NoticeTitle,
                            pntype = "p"
                        });

                    //下一条
                    var nTemp = list.Find(p => p.Page == temp.Page + 1);
                    if (nTemp != null)
                        returnDto.Add(new()
                        {
                            NoticeID = nTemp.NoticeID,
                            NoticeTitle = nTemp.NoticeTitle,
                            pntype = "n"
                        });
                    return OpenResponse.Ok(WebPortalDeResponse<List<ArticlePreviousAndNextDataDto>>.Success(returnDto));
                }
                else
                    return OpenResponse.Ok(WebPortalDeResponse<List<ArticlePreviousAndNextDataDto>>.Success(null));
            }
        }

        /// <summary>
        /// 全文检索
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getfulltextsearchlist")]
        public async Task<OpenResponse> GetFullTextSearchList([FromBody] QueryArticleListDto dto)
        {
            if (string.IsNullOrEmpty(dto.Condition))
                return OpenResponse.Ok(WebPortalDeResponse<FullTextSearchListDataDto>.Success(null));

            RefAsync<int> total = 0;
            var items = await _bulletinRepository.Queryable()
                .Where(p => p.LoseEfficacyTime >= DateTime.Now)
                 .Where(p => p.PushRanges.Contains(new() { Key = "2", Value = "门户网站" }))
                 .Where(p => p.BulletinTypeId == "1" || p.BulletinTypeId == "5" || p.BulletinTypeId == "6" || p.BulletinTypeId == "7" || p.BulletinTypeId == "3")
                 .WhereIF(!string.IsNullOrEmpty(dto.Condition), p => p.Title.Contains(dto.Condition))
                 .OrderByDescending(p => p.BulletinTime)
                 .Select(it => new
                 {
                     Page = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'NoticeCreateDate' desc ) "),
                     NoticeID = it.Id,
                     Content = it.Content,
                     NoticeTypeID = it.BulletinTypeId,
                     NoticeTypeName = it.BulletinTypeName,
                     NoticeTitle = it.Title,
                     NoticeBMName = it.SourceOrgName,
                     NoticeCreateDate = it.BulletinTime,
                     TypeCode = "",
                 })
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));
            FullTextSearchListDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<FullTextSearchListDto>>(items)
            };
            return OpenResponse.Ok(WebPortalDeResponse<FullTextSearchListDataDto>.Success(dataDto));
        }

        #endregion

        #region 办件
        /// <summary>
        /// 办件摘编列表数据
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("GetOrderList")]
        [AllowAnonymous]
        public async Task<OpenResponse> GetOrderList([FromBody] QueryOrderListDto dto)
        {
            RefAsync<int> total = 0;
            var items = await _dataOrderRepository.Queryable()
                .Where(p => p.IsPublish == EDsPublishState.Open)
                .Where(p => p.HandleState == "办理完成")
                .WhereIF(!string.IsNullOrEmpty(dto.FlowCode), d => d.OrderNo == dto.FlowCode)
                .WhereIF(!string.IsNullOrEmpty(dto.FlowName), d => d.Title.Contains(dto.FlowName))
                .WhereIF(!string.IsNullOrEmpty(dto.FlowSType), d => d.CaseTypeCode == dto.FlowSType)
                .WhereIF(!string.IsNullOrEmpty(dto.FlowRType), d => d.HotspotId == dto.FlowRType)
                .WhereIF(!string.IsNullOrEmpty(dto.FlowSDate), d => d.CaseDate >= DateTime.Parse(DateTime.Parse(dto.FlowSDate).ToString("yyyy-MM-dd 00:00:00")))//dto.FlowSDate
                .WhereIF(!string.IsNullOrEmpty(dto.FlowEDate), d => d.CaseDate <= DateTime.Parse(DateTime.Parse(dto.FlowEDate).ToString("yyyy-MM-dd 00:00:00")))// dto.FlowEDate
                .WhereIF(!string.IsNullOrEmpty(dto.FlowFrom), d => d.FromName.Contains(dto.FlowFrom))
                .OrderByDescending(p => p.PublishDate)
                .ToPageListAsync(dto.PageIndex, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));
            OrderListReturnDto returnDto = new()
            {
                PageNum = dto.PageIndex,
                PageCount = nPageCount,
                Data = _mapper.Map<IReadOnlyList<OrderListDto>>(items)
            };

            return OpenResponse.Ok(WebPortalDeResponse<OrderListReturnDto>.Success(returnDto, "成功"));
        }

        /// <summary>
        /// 办件摘编详情
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getorderdetailbyid")]
        public async Task<OpenResponse> GetOrderDetailById(string id)
        {
            var data = await _dataOrderRepository.GetAsync(p => p.OrderId == id);

            var orderDetail = _mapper.Map<OrderDetail>(data);
            return OpenResponse.Ok(WebPortalDeResponse<OrderDetail>.Success(orderDetail));
        }

        /// <summary>
        /// 根据编号和密码查询信件ID
        /// </summary>
        /// <param name="OrderNo"></param>
        /// <param name="Pwd"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getorderid")]
        public async Task<OpenResponse> GetOrderId(string OrderNo, string Pwd)
        {
            var data = await _dataOrderRepository.GetAsync(p => p.OrderNo == OrderNo && p.Password == Pwd);

            return OpenResponse.Ok(WebPortalDeResponse<string>.Success(data?.OrderId));
        }

        /// <summary>
        /// 写信接口
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("orderacceptance")]
        public async Task<OpenResponse> OrderAcceptance([FromBody] DsWebFlowAccept dto)
        {
            var data = _mapper.Map<AddOrderDto>(dto);
            data.SourceChannel = "网站";
            data.SourceChannelCode = "WZ";
            data.Source = ESource.Hotline;
            if (!string.IsNullOrEmpty(data.LicenceNo))
            {
                data.LicenceTypeCode = "10";
                data.LicenceType = "中华人民共和国居民身份证";
            }
            var result = await _mediator.Send(new ReceiveOrderNotify { AddOrderDto = data }, HttpContext.RequestAborted);

            OrderAcceptanceReturnDto returnDto = new();
            if (result != null && !string.IsNullOrEmpty(result.Id))
            {
                returnDto.PWD = result.Password;
                returnDto.Code = result.No;
                returnDto.State = "1";

                dto.Pwd = result.Password;
                dto.Code = result.No;
                dto.OrderId = result.Id;
                await _webFlowAcceptRepository.AddAsync(dto, HttpContext.RequestAborted);

            }
            else
                returnDto.State = "0";

            return OpenResponse.Ok(WebPortalDeResponse<OrderAcceptanceReturnDto>.Success(returnDto));
        }

        /// <summary>
        /// 受理类型和热点统计
        /// </summary>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getchartdata")]
        public async Task<OpenResponse> GetChartData()
        {
            var startDate = Convert.ToDateTime(DateTime.Now.AddMonths(-1).ToString("yyyy-MM-dd 00:00:00"));
            var endDate = Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd 23:59:59"));

            //数据查询
            var listType = await _dataOrderRepository.Queryable()
                .Where(p => p.CaseDate >= startDate)
                .Where(p => p.CaseDate <= endDate)
                .Select(it => new
                {
                    it.CaseType,
                    it.CaseTypeCode
                })
                .MergeTable()//将查询出来的结果合并成一个新表
                 .GroupBy(it => new { it.CaseType, it.CaseTypeCode })//对新表进行分组
                 .Select(it => new
                 {
                     name = it.CaseType,
                     value = SqlFunc.AggregateCount(it.CaseTypeCode)
                 })
                 .ToListAsync();

            //数据查询-查询总数前5的数据
            var listHot = await _dataOrderRepository.Queryable()
                .Where(p => p.CaseDate >= startDate)
                .Where(p => p.CaseDate <= endDate)
                .Select(it => new
                {
                    it.HotspotId,
                    it.HotspotName
                })
                .MergeTable()//将查询出来的结果合并成一个新表
                 .GroupBy(it => new { it.HotspotId, it.HotspotName })//对新表进行分组
                 .Select(it => new
                 {
                     typeName = it.HotspotName,
                     num = SqlFunc.AggregateCount(it.HotspotId)
                 })
                 .OrderByDescending(it => it.num)
                 .Take(5)
                 .ToListAsync();

            GetChartDataDto dataDto = new()
            {
                formCount = _mapper.Map<IReadOnlyList<OrderFormCount>>(listType),
                hotCount = _mapper.Map<IReadOnlyList<OrderHotCount>>(listHot)
            };

            return OpenResponse.Ok(WebPortalDeResponse<GetChartDataDto>.Success(dataDto));
        }

        /// <summary>
        /// 获取统计数据
        /// </summary>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("getstatist")]
        public async Task<OpenResponse> GetStatist()
        {
            var startDate = Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd 00:00:00"));
            var endDate = Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd 23:59:59"));
            GetStatistDto getStatistDto = new()
            {
                AllCount = await _dataOrderRepository.Queryable().CountAsync(),
                AllTrandCount = await _dataOrderRepository.Queryable().Where(p => p.HandleState == "办理完成").CountAsync(),
                DayCount = await _dataOrderRepository.Queryable().Where(p => p.CaseDate >= startDate).Where(p => p.CaseDate <= endDate).CountAsync(),
                DayTrandCount = await _dataOrderRepository.Queryable().Where(p => p.HandleState == "办理完成").Where(p => p.CaseDate >= startDate).Where(p => p.CaseDate <= endDate).CountAsync()
            };

            return OpenResponse.Ok(WebPortalDeResponse<GetStatistDto>.Success(getStatistDto));
        }
        #endregion

        #region 用户
        /// <summary>
        /// 添加统一认证用户数据
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [AllowAnonymous]
        [HttpPost("adduserauth")]
        public async Task<OpenResponse> AddUserAuth([FromBody] UserModelDto dto)
        {
            bool bResult = false;
            // 唯一记录ID
            string id = dto.id;
            // 注册用户ID
            string webUserID = "";
            // 统一认证用户注册ID
            string wUAID = "";
            //查询是否有注册用户
            var dataUser = await _webUserAuthRepository.GetAsync(p => p.DataId == id, HttpContext.RequestAborted);
            if (dataUser != null)
            {
                webUserID = dataUser.WebUserID;
                wUAID = dataUser.Id;
            }
            //没有注册用户添加数据
            if (string.IsNullOrEmpty(webUserID))
            {
                var userRegister = _mapper.Map<DsWebUserRegister>(dto);
                userRegister.RegDate = DateTime.Now;
                userRegister.LastDate = DateTime.Now;
                webUserID = await _webUserRegisterRepository.AddAsync(userRegister, HttpContext.RequestAborted);
            }

            //统一认证数据
            var userAuth = _mapper.Map<DsWebUserAuth>(dto);
            if (!string.IsNullOrEmpty(wUAID))
            {
                //修改数据
                userAuth.Id = wUAID;
                await _webUserAuthRepository.UpdateAsync(userAuth, HttpContext.RequestAborted);
                bResult = true;
            }
            else
            {
                //新增数据
                userAuth.WebGUID = Guid.NewGuid().ToString();
                userAuth.WebUserID = webUserID;
                var addId = await _webUserAuthRepository.AddAsync(userAuth, HttpContext.RequestAborted);
                if (string.IsNullOrEmpty(addId))
                    bResult = true;
            }
            //判断是否成功
            if (bResult)
                return OpenResponse.Ok(WebPortalDeResponse<string>.Success(webUserID));
            else
                return OpenResponse.Ok(WebPortalDeResponse<string>.Failed());
        }
        #endregion
    }
}
